@@ -17,6 +17,7 @@
 // C standard libraries
 #include <assert.h>
 #include <errno.h>
+#include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,8 +39,13 @@
 static char *milestones[] = {"created"
                             ,"release"
                             ,"eol"
+#ifdef DEBIAN
+                            ,"eol-lts"
+                            ,"eol-elts"
+#endif
 #ifdef UBUNTU
                             ,"eol-server"
+                            ,"eol-esm"
 #endif
 };
 
@@ -164,6 +170,37 @@ static date_t *read_date(const char *s, int *failures, const char *filename,
     return date;
 }
 
+// Return current date. Take SOURCE_DATE_EPOCH into account.
+// The result needs to be freed after its use.
+static date_t *get_current_date() {
+    char *source_date_epoch, *endptr;
+    date_t *date;
+    unsigned long long epoch;
+    struct tm *now;
+    time_t time_now = 0;
+
+    source_date_epoch = getenv("SOURCE_DATE_EPOCH");
+    if (source_date_epoch != NULL) {
+        epoch = strtoull(source_date_epoch, &endptr, 10);
+        if (epoch > 0 && epoch < ULONG_MAX && *endptr == '\0') {
+            time_now = epoch;
+        } else {
+            fprintf(stderr, NAME ": Environment variable SOURCE_DATE_EPOCH"
+                    "='%s' not a valid epoch. Ignoring.\n", source_date_epoch);
+        }
+    }
+    if (time_now == 0) {
+        time_now = time(NULL);
+    }
+    now = gmtime(&time_now);
+
+    date = malloc(sizeof(date_t));
+    date->year = 1900 + now->tm_year;
+    date->month = 1 + now->tm_mon;
+    date->day = now->tm_mday;
+    return date;
+}
+
 static inline bool date_ge(const date_t *date1, const date_t *date2) {
     return date1->year > date2->year ||
            (date1->year == date2->year && date1->month > date2->month) ||
@@ -193,6 +230,31 @@ static inline bool eol(const date_t *date, const distro_t *distro) {
     ;
 }
 
+#ifdef DEBIAN
+static inline bool eol_lts(const date_t *date, const distro_t *distro) {
+    return !distro->milestones[MILESTONE_EOL_LTS] ||
+           date_ge(date, distro->milestones[MILESTONE_EOL_LTS])
+    ;
+}
+
+static inline bool eol_elts(const date_t *date, const distro_t *distro) {
+    return !distro->milestones[MILESTONE_EOL_ELTS] ||
+           date_ge(date, distro->milestones[MILESTONE_EOL_ELTS])
+    ;
+}
+#endif
+
+#ifdef UBUNTU
+static inline bool eol_esm(const date_t *date, const distro_t *distro) {
+    return distro->milestones[MILESTONE_EOL] &&
+           date_ge(date, distro->milestones[MILESTONE_EOL])
+           && (!distro->milestones[MILESTONE_EOL_ESM] ||
+              (distro->milestones[MILESTONE_EOL_ESM] &&
+               date_ge(date, distro->milestones[MILESTONE_EOL_ESM])))
+    ;
+}
+#endif
+
 // Filter callbacks
 
 static bool filter_all(unused(const date_t *date),
@@ -208,6 +270,23 @@ static bool filter_supported(const date_t *date, const distro_t *distro) {
     return created(date, distro) && !eol(date, distro);
 }
 
+#ifdef DEBIAN
+static bool filter_lts_supported(const date_t *date, const distro_t *distro) {
+    return created(date, distro) && eol(date, distro) && !eol_lts(date, distro);
+}
+
+static bool filter_elts_supported(const date_t *date, const distro_t *distro) {
+    return created(date, distro) && eol_lts(date, distro) && !eol_elts(date, distro);
+}
+#endif
+
+#ifdef UBUNTU
+static bool filter_esm_supported(const date_t *date, const distro_t *distro) {
+    return created(date, distro) && !eol_esm(date, distro) &&
+           strstr(distro->version, "LTS") != NULL;
+}
+#endif
+
 static bool filter_unsupported(const date_t *date, const distro_t *distro) {
     return created(date, distro) && eol(date, distro);
 }
@@ -372,8 +451,13 @@ static void free_data(distro_elem_t *list, char **content) {
         free(list->distro->milestones[MILESTONE_CREATED]);
         free(list->distro->milestones[MILESTONE_RELEASE]);
         free(list->distro->milestones[MILESTONE_EOL]);
+#ifdef DEBIAN
+        free(list->distro->milestones[MILESTONE_EOL_LTS]);
+        free(list->distro->milestones[MILESTONE_EOL_ELTS]);
+#endif
 #ifdef UBUNTU
         free(list->distro->milestones[MILESTONE_EOL_SERVER]);
+        free(list->distro->milestones[MILESTONE_EOL_ESM]);
 #endif
         free(list->distro);
         free(list);
@@ -397,9 +481,9 @@ static distro_elem_t *read_data(const char *filename, char **content) {
     data = *content = read_full_file(filename);
     line = strsep(&data, "\n");
     lineno = 1;
-    if(unlikely(strcmp(CSV_HEADER, line) != 0)) {
-        fprintf(stderr, NAME ": Header `%s' in file `%s' does not match "
-                "excatly `" CSV_HEADER "'.\n", line, filename);
+    if(unlikely(strncmp(CSV_HEADER, line, strlen(CSV_HEADER)) != 0)) {
+        fprintf(stderr, NAME ": Header `%s' in file `%s' does not start with "
+                "`" CSV_HEADER "'.\n", line, filename);
         failures++;
     }
 
@@ -525,16 +609,23 @@ static void print_help(void) {
 #endif
            "  -a  --all              list all known versions\n"
            "  -d  --devel            latest development version\n"
+#ifdef DEBIAN
+           "  -t  --testing          current testing version\n"
+#endif
+           "  -s  --stable           latest stable version\n"
 #ifdef UBUNTU
            "      --lts              latest long term support (LTS) version\n"
 #endif
 #ifdef DEBIAN
            "  -o  --oldstable        latest oldstable version\n"
 #endif
-           "  -s  --stable           latest stable version\n"
-           "      --supported        list of all supported stable versions\n"
+           "      --supported        list of all supported versions (including development)\n"
 #ifdef DEBIAN
-           "  -t  --testing          current testing version\n"
+           "  -l  --lts              list of all LTS supported versions\n"
+           "  -e  --elts             list of all Extended LTS supported versions\n"
+#endif
+#ifdef UBUNTU
+           "      --supported-esm    list of all Ubuntu Advantage supported stable versions\n"
 #endif
            "      --unsupported      list of all unsupported stable versions\n"
            "  -c  --codename         print the codename (default)\n"
@@ -550,13 +641,20 @@ static inline int not_exactly_one(void) {
             "--alias, "
 #endif
             "--all, --devel, "
-#ifdef UBUNTU
-            "--latest, --lts, "
+#ifdef DEBIAN
+            "--elts, "
 #endif
+#ifdef UBUNTU
+            "--latest, "
+#endif
+            "--lts, "
 #ifdef DEBIAN
             "--oldstable, "
 #endif
             "--stable, --supported, "
+#ifdef UBUNTU
+            "--supported-esm, "
+#endif
             "--series, "
 #ifdef DEBIAN
             "--testing, "
@@ -590,35 +688,40 @@ int main(int argc, char *argv[]) {
 #endif
 
     const struct option long_options[] = {
-        {"help",        no_argument,       NULL, 'h' },
-        {"date",        required_argument, NULL, 'D' },
-        {"series",      required_argument, NULL, 'R' },
-        {"all",         no_argument,       NULL, 'a' },
-        {"days",        optional_argument, NULL, 'y' },
-        {"devel",       no_argument,       NULL, 'd' },
-        {"stable",      no_argument,       NULL, 's' },
-        {"supported",   no_argument,       NULL, 'S' },
-        {"unsupported", no_argument,       NULL, 'U' },
-        {"codename",    no_argument,       NULL, 'c' },
-        {"fullname",    no_argument,       NULL, 'f' },
-        {"release",     no_argument,       NULL, 'r' },
+        {"help",          no_argument,       NULL, 'h' },
+        {"date",          required_argument, NULL, 'D' },
+        {"series",        required_argument, NULL, 'R' },
+        {"all",           no_argument,       NULL, 'a' },
+        {"days",          optional_argument, NULL, 'y' },
+        {"devel",         no_argument,       NULL, 'd' },
+        {"stable",        no_argument,       NULL, 's' },
+        {"supported",     no_argument,       NULL, 'S' },
+#ifdef UBUNTU
+        {"supported-esm", no_argument,       NULL, 'e' },
+#endif
+        {"unsupported",   no_argument,       NULL, 'U' },
+        {"codename",      no_argument,       NULL, 'c' },
+        {"fullname",      no_argument,       NULL, 'f' },
+        {"release",       no_argument,       NULL, 'r' },
 #ifdef DEBIAN
-        {"alias",       required_argument, NULL, 'A' },
-        {"oldstable",   no_argument,       NULL, 'o' },
-        {"testing",     no_argument,       NULL, 't' },
+        {"alias",         required_argument, NULL, 'A' },
+        {"elts",          no_argument,       NULL, 'e' },
+        {"lts",           no_argument,       NULL, 'l' },
+        {"oldstable",     no_argument,       NULL, 'o' },
+        {"testing",       no_argument,       NULL, 't' },
 #endif
 #ifdef UBUNTU
-        {"latest",      no_argument,       NULL, 'l' },
-        {"lts",         no_argument,       NULL, 'L' },
+        {"latest",        no_argument,       NULL, 'l' },
+        {"lts",           no_argument,       NULL, 'L' },
 #endif
-        {NULL,          0,                 NULL, '\0' }
+        {NULL,            0,                 NULL, '\0' }
     };
 
 #ifdef UBUNTU
     const char *short_options = "hadscrfly::";
 #endif
 #ifdef DEBIAN
-    const char *short_options = "hadscrfoty::";
+    const char *short_options = "hadescrfloty::";
 #endif
 
     // Suppress error messages from getopt_long
@@ -728,6 +831,18 @@ int main(int argc, char *argv[]) {
 #endif
 
 #ifdef DEBIAN
+            case 'e':
+                selected_filters++;
+                filter_cb = filter_elts_supported;
+                select_cb = NULL;
+                break;
+
+            case 'l':
+                selected_filters++;
+                filter_cb = filter_lts_supported;
+                select_cb = NULL;
+                break;
+
             case 'o':
                 selected_filters++;
                 filter_cb = filter_oldstable;
@@ -753,6 +868,15 @@ int main(int argc, char *argv[]) {
                 select_cb = NULL;
                 break;
 
+#ifdef UBUNTU
+            case 'e':
+                // Only long option --supported-esm is used
+                selected_filters++;
+                filter_cb = filter_esm_supported;
+                select_cb = NULL;
+                break;
+
+#endif
 #ifdef DEBIAN
             case 't':
                 selected_filters++;
@@ -832,12 +956,7 @@ int main(int argc, char *argv[]) {
     }
 
     if(unlikely(date == NULL)) {
-        time_t time_now = time(NULL);
-        struct tm *now = gmtime(&time_now);
-        date = malloc(sizeof(date_t));
-        date->year = 1900 + now->tm_year;
-        date->month = 1 + now->tm_mon;
-        date->day = now->tm_mday;
+        date = get_current_date();
     }
 
     distro_list = read_data(DATA_DIR "/" CSV_NAME ".csv", &content);
